import { defineNuxtModule, logger } from '@nuxt/kit';
import { mergeConfig } from 'vite';
import { getVitestConfig } from 'vitest-environment-nuxt/config';
import { getPort } from 'get-port-please';

const vitePluginBlocklist = [
  "vite-plugin-vue-inspector"
];
const module = defineNuxtModule({
  meta: {
    name: "nuxt-vitest"
  },
  defaults: {
    startOnBoot: false
  },
  async setup(options, nuxt) {
    if (!nuxt.options.dev)
      return;
    if (process.env.TEST || process.env.VITE_TEST)
      return;
    const PORT = await getPort({ port: 15555 });
    const URL = `http://localhost:${PORT}/__vitest__/`;
    const viteConfig = new Promise((resolve) => {
      nuxt.hook("vite:extendConfig", (viteConfig2) => {
        getVitestConfig({ nuxt, viteConfig: viteConfig2 }).then(resolve);
      });
    }).finally(() => nuxt.close());
    async function startVitest() {
      const { startVitest: startVitest2 } = await import('vitest/node');
      const config = mergeConfig(
        await viteConfig,
        {
          server: {
            middlewareMode: false
          }
        }
      );
      config.plugins = (config.plugins || []).filter((p) => {
        return !vitePluginBlocklist.includes(p?.name);
      });
      const promise = startVitest2(
        "test",
        [],
        {
          reporters: [{}],
          ui: true,
          open: false,
          api: {
            port: PORT
          }
        },
        config
      );
      logger.info(`Vitest UI starting on ${URL}`);
      return await promise;
    }
    nuxt.hook("devtools:customTabs", (iframeTabs) => {
      iframeTabs.push({
        title: "Vitest",
        name: "vitest",
        icon: "logos-vitest",
        view: {
          type: "iframe",
          src: URL
        },
        lazy: {
          onLoad: async () => {
            await startVitest();
          },
          description: "Start tests along with Nuxt"
        }
      });
    });
    startVitest().catch((err) => {
      console.error(err);
      process.exit(1);
    });
  }
});

export { module as default };
